<?php
    $searchAppDir = (function ()
    {
        $sBaseDir = (dirname(__DIR__)).'/*';
        $sAppFolder = false;
        foreach (glob($sBaseDir, GLOB_MARK|GLOB_ONLYDIR) as $sFolder) {
            $sFolder = str_replace('\\', '/', $sFolder);
            $sConfigFile = $sFolder.'config.php';
            if (file_exists($sConfigFile)) {
                $sAppFolder = ($sFolder);
                break;
            }
        }
/* */
        if (!$sAppFolder) {
            throw new RuntimeException('sorry, '.basename($sConfigFile).' is not readable or does not exist!');
        }
        return $sAppFolder;
    });
/* ************************************************************************** */
    $sanitizeConfigFile = (function ($sConfigFile, $aDefault)
    {
        $aRetval  = [];
        $aMatches = [];
        $sFileMarker = '*** auto generated config file';
        $sCfgContent = file_get_contents($sConfigFile);

        $sPattern = '=define\s*\(\'DB_TYPE\'\,\s*\'([^\']*)\'=is';
        $sDbType  = ((preg_match($sPattern, $sCfgContent, $aMatches)) ? $aMatches[1] : 'mysqli');

        $sPattern = '=define\s*\(\'DB_HOST\'\,\s*\'([^\']*)\'=is';
        $sDbHost  = ((preg_match($sPattern, $sCfgContent, $aMatches)) ? $aMatches[1] : 'localhost');

        $sPattern = '=define\s*\(\'DB_PORT\'\,\s*\'([^\']*)\'=is';
        $sDbPort  = ((preg_match($sPattern, $sCfgContent, $aMatches)) ? $aMatches[1] : '3306');
        $aRetval['DbPort'] = $sDbPort;

        $sPattern = '=define\s*\(\'DB_NAME\'\,\s*\'([^\']*)\'=is';
        $sDbName  = ((preg_match($sPattern, $sCfgContent, $aMatches)) ? $aMatches[1] : 'wbdemo');

        $sPattern = '=define\s*\(\'DB_USERNAME\'\,\s*\'([^\']*)\'=is';
        $sDbUser  = ((preg_match($sPattern, $sCfgContent, $aMatches)) ? $aMatches[1] : 'root');

        $sPattern = '=define\s*\(\'DB_PASSWORD\'\,\s*\'([^\']*)\'=is';
        $sDbPword = ((preg_match($sPattern, $sCfgContent, $aMatches)) ? $aMatches[1] : 'usbw');

    // get db charset
        $sPattern = '=define\s*\(\'DB_CHARSET\'\,\s*\'([^\']*)\'=is';
        $sDbCharset = ((preg_match($sPattern, $sCfgContent, $aMatches)) ? $aMatches[1] : 'utf8_unicode_ci');
        $aNeedles =['utf8_unicode_ci', 'utf8mb4_unicode_ci'];
        $sDbCharset = (in_array($sDbCharset, $aNeedles)  ? $sDbCharset : 'utf8_unicode_ci');
//        $aRetval['DbCharset'] = $sDbCharset;

        $sPattern = '=define\s*\(\'TABLE_PREFIX\'\,\s*\'([^\']*)\'=is';
        $sTPrefix = ((preg_match($sPattern, $sCfgContent, $aMatches)) ? $aMatches[1] : 'wb_');

        $sPattern = '=define\s*\(\'WB_URL\'\,\s*\'([^\']*)\'=is';
        $sAppUrl  = ((preg_match($sPattern, $sCfgContent, $aMatches)) ? $aMatches[1] : 'http://localhost');
        $aRetval['AppUrl'] = $sAppUrl;

        $sPattern  = '=define\s*\(\'ADMIN_DIRECTORY\'\,\s*\'([^\']*)\'=is';
        $sAdminDir = ((preg_match($sPattern, $sCfgContent, $aMatches)) ? $aMatches[1] : 'admin');

        $aResult = array_diff($aDefault, $aRetval);
#        $bCharsetConfig = preg_match('/'.preg_quote($sDbCharset, '/').'/siU', $sCfgContent);
#        return $aRetval;

    // check if config is created by WB
        if (is_array($aRetval) && count($aRetval)) {
            if (!is_writeable($sConfigFile)) {
                $sMsg = 'The file ['.basename($sConfigFile).'] is not writable and can not be corrected!'."\n"
                      . 'Please grant the necessary rights to the file and restart this program!';
                throw new RuntimeException($sMsg);
            }
            // clean from includes
            $sPattern = '/\n[^;]*(require|include).*framework\/initialize\.php.*$/siU';
            $sCfgContent = preg_replace($sPattern, "\n",$sCfgContent);
            // create temporary file
            $sTmpFilename = tempnam(dirname(__DIR__).'/temp', '~config');
            // fill it with old content
            file_put_contents($sTmpFilename, $sCfgContent);
            // include this file
            include $sTmpFilename;
            // it can be deleted now
            unlink($sTmpFilename);
            // collect and check available data
            $aValues['DB_TYPE']      = $sDbType;
            $aValues['DB_HOST']      = $sDbHost;
            $aValues['DB_PORT']      = $aDefault['DbPort'];
            $aValues['DB_NAME']      = $sDbName;
            $aValues['DB_USERNAME']  = $sDbUser;
            $aValues['DB_PASSWORD']  = $sDbPword;
            $aValues['DB_CHARSET']   = $sDbCharset;
            $aValues['TABLE_PREFIX'] = $sTPrefix;
            $aValues['WB_URL']       = $aDefault['AppUrl'];
            $aValues['ADMIN_DIRECTORY'] = $sAdminDir;
            // build the new config content
            $sConfigContent
                = '<?php'."\n"
                . '/*'."\n"
                . ' '.$sFileMarker."\n"
                . ' ****[WebsiteBaker]****'."\n"
                . ' *** created at '.date('Y-m-d h:i:s e')."\n"
                . ' */'."\n"
                . '// define(\'DEBUG\', false);'."\n"
                . 'define(\'DB_TYPE\', \''.$aValues['DB_TYPE'].'\');'."\n"
                . 'define(\'DB_HOST\', \''.$aValues['DB_HOST'].'\');'."\n"
                . 'define(\'DB_PORT\', \''.$aValues['DB_PORT'].'\');'."\n"
                . 'define(\'DB_NAME\',  \''.$aValues['DB_NAME'].'\');'."\n"
                . 'define(\'DB_USERNAME\', \''.$aValues['DB_USERNAME'].'\');'."\n"
                . 'define(\'DB_PASSWORD\', \''.$aValues['DB_PASSWORD'].'\');'."\n"
                . 'define(\'DB_CHARSET\', \''.$aValues['DB_CHARSET'].'\');'."\n"
                . 'define(\'TABLE_PREFIX\', \''.$aValues['TABLE_PREFIX'].'\');'."\n"
                . "\n"
                . 'define(\'WB_URL\', \''.$aValues['WB_URL'].'\'); '
                . '// no trailing slash or backslash!!'."\n"
                . 'define(\'ADMIN_DIRECTORY\', \''.$aValues['ADMIN_DIRECTORY'].'\'); '
                . '// no leading/trailing slash or backslash!! A simple directory name only!!'."\n"
                . "\n"
                . 'require_once __DIR__.\'/framework/initialize.php\';'."\n"
                . '// --- end of file ----------------------------------'."\n"
            ;
            if (false === file_put_contents($sConfigFile, $sConfigContent)) {
                $sMsg = 'Write file ['.basename($sConfigFile).'] failed!'."\n"
                      . 'Please create the file manually. You can find an example at '
                      . '<a href="http://wiki.websitebaker.org/" title="WB-wiki">WebsiteBaker Wiki</a>';
                throw new RuntimeException($sMsg);
            }
            $sMsg = 'Update file ['.basename($sConfigFile).'] successfully completed!';
#            throw new RuntimeException($sMsg);
        }
    });
/* ************************************************************************** */

    $sInstallFolderAbs = $searchAppDir();
    $sInstallFolderRel =  '/'.basename($sInstallFolderAbs);
/**/
    $sProtokol = ((!isset($_SERVER['HTTPS']) || $_SERVER['HTTPS'] == 'off' ) ? 'http' : 'https') . '://';
    $sPort = in_array((int) $_SERVER['SERVER_PORT'], [80, 443]) ? '' : ':'.$_SERVER['SERVER_PORT'];
    $sHostname = str_replace($sPort, '',$_SERVER['HTTP_HOST']);
    $sAbsUrl = $sProtokol.$sHostname.$sPort;// end new routine
    $sAppUrl = $sProtokol.$sHostname.$sPort.$sInstallFolderRel;// end new routine
    $aDbPort = ini_get('mysqli.default_port');
    $aDefault = [
        'DbPort' => $aDbPort,
        'AppUrl' => $sAppUrl,
//        'DbCharset' => 'utf8_unicode_ci',
    ];

    $sConfigFile = ($sInstallFolderAbs.'config.php');
    $sanitizeConfigFile($sConfigFile, $aDefault);



/*
    $aResult = array_diff($aDefault, $aMatches);
print '<pre  class="mod-pre rounded">function <span>'.__FUNCTION__.'( '.''.' );</span>  filename: <span>'.basename(__FILE__).'</span>  line: '.__LINE__.' -> <br />';
print_r( $aDefault ); print '</pre>'; flush (); //  ob_flush();;sleep(10); die();
print '<pre  class="mod-pre rounded">function <span>'.__FUNCTION__.'( '.''.' );</span>  filename: <span>'.basename(__FILE__).'</span>  line: '.__LINE__.' -> <br />';
print_r( $aMatches ); print '</pre>'; flush (); //  ob_flush();;sleep(10); die();
print '<pre  class="mod-pre rounded">function <span>'.__FUNCTION__.'( '.''.' );</span>  filename: <span>'.basename(__FILE__).'</span>  line: '.__LINE__.' -> <br />';
print_r( $aResult ); print '</pre>'; flush (); //  ob_flush();;sleep(10); die();
#    $sScriptPath = str_replace('\\', '/', dirname($_SERVER['SCRIPT_FILENAME']));
#    $sScriptUrl = $sUrl.str_replace($wb_path, '', $sScriptPath);
*/
?><!DOCTYPE HTML>
<html lang="<?php echo strtolower($lang); ?>">
<head>
  <meta charset="utf-8" />
  <!--[if IE]><meta http-equiv='X-UA-Compatible' content='IE=edge,chrome=1'><![endif]-->
  <title><?php echo $title; ?></title>
  <link href="screen.css" rel="stylesheet" type="text/css" />
</head>
<body>
<a id="top"></a>
<div id="framers_framer">
        <div id="header">
            <div class="logo">
                <a target="_self" href="<?php echo $sAbsUrl;?>/docs/"><img src="1x1.gif" alt="" width="567" height="159" /></a></div>
            <div class="print_logo" style="display: none;">
                <img src="1x1.gif" alt="" width="477" height="229" />
                <p>WebsiteBaker CMS - Portable Edition</p>
            </div>
            <div class="about_pic pic_<?php echo $lang; ?>">&nbsp;</div>
        </div>
    <!-- /#header -->

    <div id="content_framer">
        <div id="bars">
            <div class="breadcrumb_bar">
            <?php if ($title != 'START - WebsiteBaker CMS Portable') { ?>
                <div>
                    <a href="en.php" target="_self" title="EN"><img class="flag_icon" src="EN.png" alt="EN" /></a>
                        &nbsp;
                    <a href="de.php" target="_self" title="DE"><img class="flag_icon" src="DE.png" alt="DE" /></a>
                        &nbsp;
                    <a href="nl.php" target="_self" title="NL"><img class="flag_icon" src="NL.png" alt="NL" /></a>
                        &nbsp;
                </div>
                <div class="breadcrumb"><strong><?php echo $crumb; ?>:</strong><span class="arrow">&nbsp;</span><span><?php echo $title; ?></span>
                    <p style="clear:both;"></p>
                </div>
            <?php } ?>
            </div>
        </div>

        <!-- /#bars -->
        <div id="sidebar">
            <ul class="leftmenu">
                <li><a href="index.php" >Start</a>
                <ul class="leftmenu">
                    <li><a title="English Help Page" href="en.php" >English</a></li>
                    <li><a title="Deutsche Hilfeseite" href="de.php" >Deutsch</a></li>
                    <li><a title="Dutch Help Page" href="nl.php" >Nederland</a></li>
                </ul>
            </li>
            </ul>
            <div>
                <h2>Useful Links</h2>
                <hr />
                <ul class="leftmenu">
                    <li><a href="https://websitebaker.org/" target="_blank" rel="noopener noreferrer" >WebsiteBaker CMS</a></li>
                    <li><a href="https://wiki.websitebaker.org/doku.php" target="_blank" rel="noopener noreferrer">WebsiteBaker Documentation</a></li>
                    <li><a href="https://forum.websitebaker.org/" target="_blank" rel="noopener noreferrer" >WebsiteBaker Forum</a></li>
                    <li><a href="https://portable.websitebaker.org/" target="_blank" rel="noopener noreferrer" >WebsiteBaker Portable</a></li>
                    <li><a href='https://getfirebug.com/' target='_blank' rel="noopener noreferrer" >Firebug.com</a></li>
                    <li><a href='https://adminer.org/' target='_blank' rel="noopener noreferrer" >MySQL Adminer</a></li>
                    <li><a href='https://www.phpmyadmin.net/' target='_blank' rel="noopener noreferrer" >MySQL phpMyAdmin</a></li>
                    <li><a href='https://notepad-plus-plus.org/' target='_blank' rel="noopener noreferrer" >Notepad++ Editor</a></li>
                </ul>
            </div>
            <div>
<?php if (!empty($TEXT['PORT_DONATE'])){?>
                <h2>Support WebsiteBaker</h2>
                <hr />
<?php echo $TEXT['PORT_DONATE']; ?>
                <br />
<?php } ?>
                <div class="mbox fb-link gradient">
                    <a class="" title="WebsiteBaker Portable on Facebook" target="_blank" rel="noopener noreferrer" href="http://www.facebook.com/WebsiteBaker.Portable">
                      <span >Visit WebsiteBaker Portable</span><p >on <span >Facebook</span></p>
<!--                    <img width="120" alt="find us on facebook" title="WebsiteBaker Portable on Facebook" src="find-on-fb.png" />-->
                    </a>
                </div>
                <div class="mbox fb-link gradient">
                    <a class="" href="https://twitter.com/websitebaker" title="WebsiteBaker on Twitter" target="_blank" rel="noopener noreferrer">
                      <span >Visit WebsiteBaker</span><p >on <span >Twitter</span></p>
                    </a>
                </div>

            </div>
            <!--<div id="copyright">
                <h3>Copyright &copy; 2013 <br /><br /><a href="http://www.websitebaker-portable.com/" target="_blank" rel="noopener noreferrer" >WebsiteBaker Portable Project</a></h3>
                <p>Text and images of this page are licensed under a <a href="http://creativecommons.org/licenses/by-nc-nd/3.0/" target="_blank" rel="noopener noreferrer">Creative Commons Attribution-NonCommercial-No Derivative 3.0</a> Licence. You are free to copy and distribute this work for noncommercial purposes as long as no changes are applied and this copyright notice and a backlink to <a href="http://www.websitebaker.org">WebsiteBaker.org</a> are provided.</p>
            </div>-->
        </div>
        <div id="content">
<?php // here begins your content
